@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
-#$Author: andrius $
-#$Date: 2020-12-22 04:44:09 -0500 (Tue, 22 Dec 2020) $ 
-#$Revision: 7142 $
-#$URL: svn://saulius-grazulis.lt/restful/tags/v0.15.2/lib/RestfulDB/SQL.pm $
+#$Author: saulius $
+#$Date: 2021-08-31 09:23:50 -0400 (Tue, 31 Aug 2021) $ 
+#$Revision: 7584 $
+#$URL: svn://saulius-grazulis.lt/restful/tags/v0.16.0/lib/RestfulDB/SQL.pm $
 #------------------------------------------------------------------------------
 #*
 #  An object to encapsulate a database connection and meta-table
@@ -25,7 +25,6 @@ our @EXPORT_OK = qw(
 );
 
 use Data::UUID;
-use Digest::MD5 qw(md5_hex);
 use DBI;
 use File::Basename qw(basename);
 use List::MoreUtils qw(uniq);
