@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
 #$Author: andrius $
-#$Date: 2020-12-22 04:44:09 -0500 (Tue, 22 Dec 2020) $ 
-#$Revision: 7142 $
-#$URL: svn://saulius-grazulis.lt/restful/tags/v0.15.2/lib/RestfulDB/Schema/DBI.pm $
+#$Date: 2022-05-04 10:01:31 -0400 (Wed, 04 May 2022) $ 
+#$Revision: 7627 $
+#$URL: svn://saulius-grazulis.lt/restful/tags/v0.16.0/lib/RestfulDB/Schema/DBI.pm $
 #------------------------------------------------------------------------------
 #*
 #  An object to metadata queries for the RestfulDB interface when the
@@ -33,8 +33,10 @@ use RestfulDB::SQL qw(
 );
 
 use Memoize;
-#memoize('is_view');
 memoize('get_tables_info');
+memoize('_get_table_attributes');
+
+use version;
 
 our $date_re = q/[0-9]{4}-[01][0-9]-[0123][0-9]/;
 our $uuid_re = q/[a-f0-9]{8}-(?:[a-f0-9]{4}-){3}[a-f0-9]{12}/;
@@ -105,7 +107,7 @@ sub _load_column_types_lengths
         while( my $row = $sth->fetchrow_hashref() ) {
             next if is_internal_SQLite_table( $row->{TABLE_NAME} );
             $db->{column_types}{$row->{TABLE_NAME}}{$row->{COLUMN_NAME}} =
-                $row->{TYPE_NAME};
+                uc $row->{TYPE_NAME};
 
             # SQLite3 stores all integers in 8-byte signed integers, thus
             # we don't check their length.
@@ -118,7 +120,7 @@ sub _load_column_types_lengths
             my $row = $column_info->{$column_name};
 
             $db->{column_types}{$db_table}{$row->{COLUMN_NAME}} =
-                $row->{TYPE_NAME};
+                uc $row->{TYPE_NAME};
 
             # For numerical fields 'length' stands for maximum absolute
             # value.
@@ -271,6 +273,9 @@ sub get_column_properties
             mimetype => {},
             measurement_units => {},
             printf_format => {},
+            sha1   => {},
+            sha256 => {},
+            sha512 => {},
             visualisation => {},
             validation_regex => {},
             can_be_suggested => {},
@@ -482,7 +487,7 @@ sub get_column_formats
 # Returns a list of unique columns in a table
 sub get_unique_columns
 {
-    my( $db, $table ) = @_;
+    my( $db, $table, $uniq_composite ) = @_;
 
     my $dbh = $db->{db}{content}{dbh};
 
@@ -500,10 +505,16 @@ sub get_unique_columns
         while( my $row = $sth->fetchrow_hashref ) {
             push @{$columns->{$row->{Key_name}}}, $row->{Column_name};
         }
-
-        return sort map { $columns->{$_}[0] }
-                    grep { scalar @{$columns->{$_}} == 1 }
-                         keys %$columns;
+        
+        if( defined $uniq_composite && $uniq_composite ){
+            return sort map { $columns->{$_}[0] }
+                        grep { scalar @{$columns->{$_}} >= 1 }
+                             keys %$columns;
+        } else {
+             return sort map { $columns->{$_}[0] }
+                        grep { scalar @{$columns->{$_}} == 1 }
+                             keys %$columns;
+        }
     } else {
         # FIXME: this method of unique column determination is unstable as
         # statistics_info() is experimental and is subject of change.
@@ -518,11 +529,20 @@ sub get_unique_columns
             next if $column->{NON_UNIQUE};
             push @{$seen_indexes->{$column->{INDEX_NAME}}}, $column_name;
         }
-        for my $index (keys %$seen_indexes) {
-            next if @{$seen_indexes->{$index}} == 1;
-            for my $column (@{$seen_indexes->{$index}}) {
-                delete $unique_columns->{$column};
+        if( defined $uniq_composite && $uniq_composite ){
+           for my $index (keys %$seen_indexes) {
+                next if @{$seen_indexes->{$index}} >= 1;
+                for my $column (@{$seen_indexes->{$index}}) {
+                    delete $unique_columns->{$column};
+                }
             }
+        } else {
+            for my $index (keys %$seen_indexes) {
+                next if @{$seen_indexes->{$index}} == 1;
+                for my $column (@{$seen_indexes->{$index}}) {
+                    delete $unique_columns->{$column};
+                }
+            }       
         }
         my @unique_columns = grep { $unique_columns->{$_}{NON_UNIQUE} == 0 }
                                   keys %$unique_columns;
@@ -747,34 +767,57 @@ sub _group_composite_keys
     return @groups;
 }
 
+# Read the 'format' and 'fmttable' columns from the RestfulDB metadata
+# table 'fk_format' and cache results for all tables:
+
+sub load_fk_formats_and_fmttables
+{
+    my( $db ) = @_;
+
+    if( !exists $db->{fk_formats_read} ) {
+        if( grep { $_ eq 'fk_format' }
+            Database::_get_table_list( $db->{db}{meta}{dbh} ) ) {
+            # Let's only read the 'fk_format' table if it exists in
+            # teh metadata, to avoid triggering unnecessary
+            # exceptions:
+            my $dbh = $db->{db}{meta}{dbh};
+            my $delim = $db->{db}{meta}{delim};
+            my $sql_statement = 'SELECT * FROM fk_format WHERE dbname = ?';
+            my @dbnames = ( $db->{db}{content}{DB} );
+            if( $db->{db}{content}{engine} ne 'mysql' ) {
+                $sql_statement .= ' OR dbname = ?';
+                push @dbnames, basename $db->{db}{content}{DB};
+            }
+            print STDERR "$sql_statement\n" if $debug_sql;
+            my $sth = $dbh->prepare( $sql_statement ) or
+                $db->error( "could not prepare SELECT statement: $DBI::errstr" );
+            $sth->execute( @dbnames ) or
+                $db->error( "Execution of $sql_statement failed: $DBI::errstr" );
+            while( my $row = $sth->fetchrow_hashref ) {
+                $db->{fk_formats}{$row->{dbtable}} = $row->{format};
+                $db->{fmttable}{$row->{dbtable}} = $row->{fmttable};
+            }
+        } else {
+            $db->{fk_formats} = {};
+            $db->{fmttable} = {};
+        }
+        $db->{fk_formats_read} = 1;
+    }
+}
+
+sub get_fk_fmttable
+{
+    my( $db, $table ) = @_;
+    return if !$db->{db}{meta}{dbh};
+    $db->load_fk_formats_and_fmttables();
+    return $db->{fmttable}{$table};    
+}
+
 sub get_fk_format
 {
     my( $db, $table ) = @_;
     return if !$db->{db}{meta}{dbh};
-    return $db->{fk_formats}{$table} if exists $db->{fk_formats};
-
-    if( !grep { $_ eq 'fk_format' }
-              Database::_get_table_list( $db->{db}{meta}{dbh} ) ) {
-        $db->{fk_formats} = {};
-        return;
-    }
-
-    my $dbh = $db->{db}{meta}{dbh};
-    my $delim = $db->{db}{meta}{delim};
-    my $sql_statement = 'SELECT * FROM fk_format WHERE dbname = ?';
-    my @dbnames = ( $db->{db}{content}{DB} );
-    if( $db->{db}{content}{engine} ne 'mysql' ) {
-        $sql_statement .= ' OR dbname = ?';
-        push @dbnames, basename $db->{db}{content}{DB};
-    }
-    print STDERR "$sql_statement\n" if $debug_sql;
-    my $sth = $dbh->prepare( $sql_statement ) or
-        $db->error( "could not prepare SELECT statement: $DBI::errstr" );
-    $sth->execute( @dbnames ) or
-        $db->error( "Execution of $sql_statement failed: $DBI::errstr" );
-    while( my $row = $sth->fetchrow_hashref ) {
-        $db->{fk_formats}{$row->{dbtable}} = $row->{format};
-    }
+    $db->load_fk_formats_and_fmttables();
     return $db->{fk_formats}{$table};
 }
 
@@ -1337,6 +1380,82 @@ sub get_db_username
     return $username;
 }
 
+sub get_table_name
+{
+    my( $db, $db_table ) = @_;
+
+    my $table_attributes = $db->_get_table_attributes();
+
+    my $table_name = defined $table_attributes->{$db_table}{'name'} ?
+                             $table_attributes->{$db_table}{'name'} :
+                             $db_table;
+
+    return $table_name;
+}
+
+sub get_table_description
+{
+    my( $db, $db_table ) = @_;
+
+    my $table_attributes = $db->_get_table_attributes();
+
+    my $table_description = defined $table_attributes->{$db_table}{'description'} ?
+                             $table_attributes->{$db_table}{'description'} :
+                             "";
+
+    return $table_description;
+}
+
+sub get_table_explanation
+{
+    my( $db, $db_table ) = @_;
+
+    my $table_attributes = $db->_get_table_attributes();
+    my $table_explanation= defined $table_attributes->{$db_table}{'explanation'} ?
+                             $table_attributes->{$db_table}{'explanation'} :
+                             "";
+
+    return $table_explanation;
+}
+
+sub _get_table_attributes
+{
+    my( $db ) = @_;
+
+    
+    my $current_metatables_version = $db->metatables_version();
+    if( defined $current_metatables_version ){
+    
+        $current_metatables_version =~ s/-.*//;
+        return undef if version->parse( $current_metatables_version ) < version->parse( '0.2.5' );
+
+        my $dbh = $db->{db}{meta}{dbh};
+        my $sql_statement = 'SELECT '. 
+                                'dbtable, name, description, explanation ' .
+                                'FROM  table_description';
+        print STDERR "$sql_statement\n" if $debug_sql;
+        my $sth = $dbh->prepare( $sql_statement ) or
+            $db->error( "could not prepare SELECT statement: $DBI::errstr" );
+
+        $sth->execute() or
+            $db->error( "'SELECT *' failed: " . $DBI::errstr );
+
+        my %table_attributes;
+
+        while(my $attributes = $sth->fetchrow_arrayref) {
+            $table_attributes{$attributes->[0]} = 
+                { name => $attributes->[1],
+                  description => $attributes->[2],
+                  explanation => $attributes->[3]
+                };
+        }
+
+        return \%table_attributes;
+    } else {
+        return undef;
+    }
+}
+
 sub metatables_version
 {
     my( $db ) = @_;
