@@ -189,12 +189,12 @@ sub read_spreadsheet
                             # Processing a date. Contains a workaround for
                             # https://projects.ibt.lt/repositories/issues/583
                             if( looks_like_number( $value ) ) {
-                                $value = strftime( "%F %T",
+                                $value = strftime( '%F %T',
                                                    ExcelLocaltime( $value ) );
-                            } else {
-                                warn "Non-numeric date value '$value' " .
-                                     'encountered in XLSX file, leaving as is.';
                             }
+                            # Other values are passed through without any
+                            # conversion or warnings. Format checks for
+                            # dates are performed elsewhere.
                         }
                     }
                 } else {
