@@ -1,8 +1,8 @@
 -- ------------------------------------------------------------------------------
--- $Author: vantanas $
--- $Date: 2020-05-26 06:12:37 -0400 (Tue, 26 May 2020) $ 
--- $Revision: 6875 $
--- $URL: svn://saulius-grazulis.lt/restful/tags/v0.15.2/sql/metatables/metatables_0.2.1.sql $
+-- $Author: saulius $
+-- $Date: 2021-03-19 12:35:48 -0400 (Fri, 19 Mar 2021) $ 
+-- $Revision: 7324 $
+-- $URL: svn://saulius-grazulis.lt/restful/tags/v0.16.0/sql/metatables/metatables_0.2.3.sql $
 -- ------------------------------------------------------------------------------
 
 -- The metatables describe fields (columns) in other tables of other
@@ -12,6 +12,15 @@
 
 -- == CHANGELOG ==
 --
+-- metatables.sql (0.2.3)
+--  * Added:
+--    - support for 'sha1', 'sha256' and 'sha512' column types.
+--  * Changed:
+--    - the 'numbers'.'range' column is now text (to accommodate long suggestions).
+-- metatables.sql (0.2.2)
+--  * Added:
+--    - the 'table_description' table;
+--    - the 'printf_format' column in the 'description' table.
 -- metatables.sql (0.2.1)
 --  * Added:
 --    - the 'measurement_unit' table;
@@ -51,8 +60,16 @@ create table `version` (
 );
 
 insert into `version` ( `id`, `release_date`, `number`, `schema_url` )
-values ( '1', '2020-05-26', '0.2.1',
-'svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/restfuldb/trunk/sql/metatables/metatables_0.2.1.sql' );
+values ( '1', '2021-03-19', '0.2.3',
+'svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/restfuldb/trunk/sql/metatables/metatables_0.2.3.sql' );
+
+create table `table_description` (
+       `id` integer primary key,
+       `dbname` varchar(128),
+       `dbtable` varchar(128),
+       `is_read_only` integer,
+       constraint `UQ_table_description_dbtable` unique (dbname,dbtable)
+);
 
 create table `measurement_unit` (
        `id` integer primary key,
@@ -61,7 +78,11 @@ create table `measurement_unit` (
 );
 
 -- Regular expressions are used to impose stricter requirements on
--- field values:
+-- field values.
+
+-- Regular expressions MUST NOT contain string start or end anchors
+-- (trailing '^' and leading '$'), as the regular expression is assumed
+-- to match the whole input (metasymbols added automatically).
 
 create table validation_regex (
        id integer primary key,
@@ -147,6 +168,18 @@ create table validation_regex (
 --      column; the target column name is specified in the 'fk_target'
 --      field of the 'description' table.
 
+-- sha1: the described column contains the sha1 checksum of another
+--       column; the target column name is specified in the 'fk_target'
+--       field of the 'description' table.
+
+-- sha256: the described column contains the sha256 checksum of another
+--         column; the target column name is specified in the 'fk_target'
+--         field of the 'description' table.
+
+-- sha512: the described column contains the sha512 checksum of another
+--         column; the target column name is specified in the 'fk_target'
+--         field of the 'description' table.
+
 -- filename: The file name that was uploaded to another ('file')
 --           column; the 'fk_target' field of the 'description' table
 --           specifies what column it is.
@@ -160,18 +193,6 @@ create table validation_regex (
 -- FIXME(S.G.): The following column descriptors are not yet
 --              implemented but are planned for the future:
 
--- sha1: the described column contains the sha1 checksum of another
---       column; the target column name is specified in the 'fk_target'
---       field of the 'description' table.
-
--- sha256: the described column contains the sha256 checksum of another
---         column; the target column name is specified in the 'fk_target'
---         field of the 'description' table.
-
--- sha512: the described column contains the sha512 checksum of another
---         column; the target column name is specified in the 'fk_target'
---         field of the 'description' table.
-
 -- script: The column value is a parameter of a generic script that
 --         renders its value on a HTML page. (A.V.: sounds
 --         dangerous...). Values must be sanitised properly!
@@ -196,6 +217,7 @@ create table description (
        -- display enum('always','notnull','never','foruser'),
        display char(7),
        measurement_unit_id integer,
+       printf_format varchar(1024),
        constraint coltype unique (dbname,dbtable,dbcolumn,coltype),
        foreign key (fk_target) references description(id),
        foreign key (validation_regex_id) references validation_regex(id),
@@ -317,7 +339,7 @@ create table fk_format (
 create table numbers (
        id integer primary key,
        column_id integer not null, -- column with suggested value
-       `range` varchar(12) not null,
+       `range` text not null,
        user varchar(255), -- name of the user associated with the
                           -- range, NULL values mark ranges common to
                           -- all users
