@@ -49,4 +49,4 @@ ${INPUT_SCRIPT}
 
 sqlite3 -nullvalue NULL ${TMP_DB_FILE} 'select * from book order by id';
 
-rm -f ${TMP_TEST_DB}
+rm -f ${TMP_DB_FILE}
