@@ -44,6 +44,7 @@ use HTMLGenerator qw(
     start_form_QS
     start_root
     end_root
+    get_foreign_key_cell_text
     header_db_pl
     start_doc_wrapper
     end_doc_wrapper
@@ -58,6 +59,7 @@ use HTMLGenerator qw(
     button_classes_string
     warnings2html
     datalists
+    table_explanation
 );
 use RestfulDB::CGI qw( save_cgi_parameters );
 use RestfulDB::Defaults qw( get_css_styles get_default_cgi_parameters );
@@ -66,6 +68,7 @@ use RestfulDB::Exception;
 use RestfulDB::JSON qw( data2json error2json json2data );
 use RestfulDB::JSONAPI qw( data2collection error2jsonapi jsonapi2data );
 use RestfulDB::Spreadsheet qw( data2spreadsheet data2template spreadsheet2data );
+use RestfulDB::SQL qw( is_blob is_text );
 
 # STDIN must NOT be set to binmode "UTF-8", since CGI with the '-utf8'
 # flag handles this.
@@ -109,12 +112,16 @@ eval {
         search_value        => { re => '.*', default => '' }, # Should be SQL-escaped by the DBI layer
         filter              => { re => '.*' }, # processed afterwards
 
+        ajax            => { re => 'select2'},
         columns         => { re => '([\w\d_]+\.?)(,[\w\d_]+\.?)*', default => '' },
         columns_add     => { re => '([\w\d_]+)(,[\w\d_]+)*', default => '' },
         columns_remove  => { re => '([\w\d_]+)(,[\w\d_]+)*', default => '' },
         depth           => { re => '-1|[0-9]+|auto', default => 0 },
+        fk_column       => { re => '([\w\d_]+)(,[\w\d_]+)*'},
         format          => { re => 'html|csv|csv[ \+]?zip|ods|xlsx?|xml|cif|json(?:api)?' },
+        page            => { re => '[0-9]+' },
         rows_template   => { re => '[0-9]+' },
+        search          => { re => '([\w\d_\s-]+)*'},
         upload_format   => { re => 'autodetect|csv|csv[ \+]?zip|ods|xlsx?' },
         include         => { re => '.*' }, # Reserved for JSONAPI use
         action          => { re => '.*' },
@@ -358,16 +365,6 @@ eval {
                         -value => 'Save',
                         -class => button_classes_string(),
                         -onclick => 'clearView(); return validate_and_submit( this.form )' );
-            
-            my $fk_columns  = $db->get_foreign_keys($db_table);
-            my $fk_table_data = $db->get_fks_cell_text( $fk_columns, 1 );
-            my $ambiguous_fks = {};
-
-            foreach my $fk( sort keys %{ $fk_table_data->{fk_values} } ){
-                my $fk_table_values = $fk_table_data->{fk_values}{$fk};
-                $ambiguous_fks->{ $fk } =
-                HTMLGenerator::get_ambiguous_fk_values($fk_table_values, $fk, $fk_columns);
-            }
 
             my $new_record_html =
                 data2html_new_form(
@@ -388,8 +385,7 @@ eval {
                             },
                             form_head => $save_button,
                             form_tail => br . $save_button,
-                            ambiguous_fks => $ambiguous_fks
-            } );
+                        } );
 
             # Collect html variables set to print.
             my $column_left_html;
@@ -450,7 +446,8 @@ eval {
 
             # The beginning of the HTML page.
             $html .= start_root();
-            $html .= header_db_pl( $db_name, $db_table );
+            $html .= header_db_pl( $db_name, $db_table,
+                                   { 'is_view' => $db->is_view( $db_table ) } );
             $html .= start_doc_wrapper();
             $html .= start_main_content();
 
@@ -851,34 +848,165 @@ eval {
         } elsif( defined $format && $format =~ /^json(api)?$/ ) {
             $params{depth} =  0 if $format eq 'json' && !exists $params{depth};
             $params{depth} = -1 if $format eq 'jsonapi';
-            my $record_descriptions =
-                $db->get_record_descriptions( $db_table,
-                                              {
-                                                  %$get_record_descriptions_options,
-                                                  depth => $params{depth},
-                                              } );
-            if( $format eq 'json' ) {
-                print $cgi->header( -type => $RestfulDB::JSON::MEDIA_TYPE,
-                                    -expires => 'now',
-                                    -charset => 'UTF-8',
-                                    -attachment => "$db_table.json" ),
-                      data2json( $db, $record_descriptions, $db_table,
-                                 { request_uri => $ENV{REQUEST_URI},
-                                   data_matched =>
-                                    $db->get_count( $db_table, $filter ),
-                                   web_base => dirname( url() ),
-                                   warnings => \@warnings } ),
-                      "\n";
+            
+            
+            my $record_descriptions;
+
+            if ( defined $params{ajax} && $params{ajax} eq 'select2' ) {
+                my $next_offset = $params{page} * $params{rows};
+                my $fk_list     = $db->get_foreign_keys($db_table);
+                my ($fk_column) = grep $_->name() eq $params{fk_column},
+                  @{$fk_list};
+                my $parent_table = $fk_column->parent_table();
+                my $fmttable = $db->get_fk_fmttable( $parent_table );
+                my $fk_display_table = defined $fmttable ?
+                    $fmttable : $parent_table;
+
+
+                my $column_properties =
+                  $db->get_column_properties( $fk_display_table );
+                my $column_sql_types =
+                  $db->get_column_type_hash( $fk_display_table );
+                my @unique_fk_table_columns =
+                  $db->get_unique_columns( $fk_display_table );
+                my $fk_format = 
+                  $db->get_fk_format( $fk_display_table );
+                my $dropdown_list = [];
+
+                my $search_columns = [
+                    grep {
+                        (       !$column_properties->{coltype}{$_}
+                              || $column_properties->{coltype}{$_} !~
+                              /^(fk|uuid|dbrev|cssclass|mimetype|format)$/ )
+                          && !is_text( $column_sql_types->{$_} )
+                          && !is_blob( $column_sql_types->{$_} )
+                    } $db->get_column_names(
+                        $fk_display_table,
+                        { display => 'all' }
+                    )
+                ];
+
+               my $search_values =
+                  [ map { $params{search} } @{$search_columns} ];
+
+
+                if ( $params{search} ) {
+                    $filter = Database::Filter->new_for_multiple_clause(
+                        [ map { [$_] } @{$search_columns} ],
+                        $search_values,
+                        {
+                            comparison     => 'CONTAINS',
+                            logic_operator => 'OR'
+                        }
+                    );
+                }
+                else {
+                    $filter = undef;
+                }
+
+                my $fk_records = $db->get_records(
+                    $fk_display_table,
+                    undef,
+                    {
+                        filter => $filter,
+                        rows   => $params{rows},
+                        offset => $next_offset
+
+                    }
+                );
+
+                my $fk_table_id_col =
+                  $db->get_id_column( $fk_display_table );
+
+                foreach my $fk ( @{$fk_records} ) {
+                    my $reversed_data;
+                    $reversed_data->{columns} = {
+                        map {
+                            $_ => {
+
+                                'value'   => $fk->{$_},
+                                'coltype' => $column_properties->{coltype}{$_},
+                                'sqltype' => $column_sql_types->{$_},
+                            }
+
+                        } keys %{$fk}
+                    };
+
+                    $reversed_data->{metadata}{fk_format} = $fk_format;
+                    $reversed_data->{value} = $fk;
+                    
+                    my $cell_text = HTMLGenerator::get_foreign_key_cell_text(
+                        $params{fk_column},
+                        $fk,
+                        {
+                            record       => $reversed_data,
+                            foreign_keys => $fk_list,
+                            ambiguity => $db->is_uniq_fk($fk_column)
+                        }
+                    );
+
+                    push(
+                        @{$dropdown_list},
+                        {
+                            text => $cell_text,
+                            id   => $fk->{$fk_table_id_col},
+                        }
+                    );
+                }
+                $record_descriptions = { data => $dropdown_list };
+
+                print $cgi->header(
+                    -type       => $RestfulDB::JSON::MEDIA_TYPE,
+                    -expires    => 'now',
+                    -charset    => 'UTF-8',
+                    -attachment => "$db_table.json"
+                  ),
+
+                  #encode_json(
+                  JSON->new()->encode(
+                    {
+                        data       => $dropdown_list,
+                        pagination => {
+                            more => $db->get_count( $fk_display_table,
+                                $filter ) < ( $next_offset + $params{rows} )
+                            ? \0
+                            : \1
+                        }
+                    }
+                  ),
+                  "\n";
+
             } else {
-                print $cgi->header( -type => $RestfulDB::JSONAPI::MEDIA_TYPE,
-                                    -expires => 'now',
-                                    -charset => 'UTF-8',
-                                    -attachment => "$db_table.json" ),
-                      data2collection( $db, $record_descriptions, $db_table,
-                                       { request_uri => $ENV{REQUEST_URI},
-                                         web_base => dirname( url() ),
-                                         warnings => \@warnings } ),
-                      "\n";
+                $record_descriptions =
+                    $db->get_record_descriptions( $db_table,
+                                                  {
+                                                      %$get_record_descriptions_options,
+                                                      depth => $params{depth},
+                                                  } );
+
+                if( $format eq 'json' ) {
+                    print $cgi->header( -type => $RestfulDB::JSON::MEDIA_TYPE,
+                                        -expires => 'now',
+                                        -charset => 'UTF-8',
+                                        -attachment => "$db_table.json" ),
+                          data2json( $db, $record_descriptions, $db_table,
+                                     { request_uri => $ENV{REQUEST_URI},
+                                       data_matched =>
+                                        $db->get_count( $db_table, $filter ),
+                                       web_base => dirname( url() ),
+                                       warnings => \@warnings } ),
+                          "\n";
+                } else {
+                    print $cgi->header( -type => $RestfulDB::JSONAPI::MEDIA_TYPE,
+                                        -expires => 'now',
+                                        -charset => 'UTF-8',
+                                        -attachment => "$db_table.json" ),
+                          data2collection( $db, $record_descriptions, $db_table,
+                                           { request_uri => $ENV{REQUEST_URI},
+                                             web_base => dirname( url() ),
+                                             warnings => \@warnings } ),
+                          "\n";
+                }
             }
         } elsif( !defined $format || $format eq 'html' ) {
             my $html;
@@ -912,13 +1040,17 @@ eval {
             # The beginning of the HTML page.
             # Open Main containers.
             $html .= start_root();
-            $html .= header_db_pl( $db_name, $db_table );
+            $html .= header_db_pl( $db_name, $db_table,
+                                   { 'is_view' => $db->is_view( $db_table ),
+                                     'table_name' => $db->get_table_name( $db_table) } );
             $html .= start_doc_wrapper();
             $html .= start_main_content();
 
             # Hidden 'top' paragraph on the data page for the 'To top' link.
             $html .= p( {-id => "top"}, "Top" ) . "\n";
 
+            $html .= table_explanation($db->get_table_explanation($db_table));
+
             # Warnings
             $html .= warnings2html( @warnings );
 
@@ -1009,52 +1141,142 @@ eval {
                 -method => 'get',
                 -class => 'form_init' ) . "\n";
 
-            my $fk_columns  = $db->get_foreign_keys($db_table);
-            my $fk_list = [ map{ $_->name() ne 'revision_id' ?
-                                    $_->name() : () } @{$fk_columns} ];
-            
-            my $fk_json = encode_json($fk_list);
+            my $fk_columns = $db->get_foreign_keys($db_table);
+            my $fk_list    = [
+                map {
+                         $_->name() ne 'revision_id'
+                      && $db->get_count( $_->parent_table() ) > 100
+                      ? $_->name()
+                      : ()
+                } @$fk_columns
+            ];
+            my $all_fk_list =
+              [ map { $_->name() ne 'revision_id' ? $_->name() : () }
+                  @$fk_columns ];
 
-            # my $fks_cell_text = encode_json($db->get_fks_cell_text($fk_columns) );
+            my $fk_json     = encode_json($fk_list);
+            my $all_fk_json = encode_json($all_fk_list);
 
             do {
                 local $" = "\n";
-                $html .= popup_menu( -name => 'select_column',
-                                     -values => \@columns,
-                                     -default => $default_select_column,
-                                     -onchange => "search_suggest($fk_json)" ) . "\n" .
+                $html .= popup_menu(
+                    -name     => 'select_column',
+                    -values   => \@columns,
+                    -default  => $default_select_column,
+                    -onchange => "search_suggest($all_fk_json, $fk_json)"
+                  )
+                  . "\n"
+                  .
 
-                         popup_menu( -name => 'select_not_operator',
-                                     -values => [ '', 'not' ],
-                                     -labels => { '' => '', 'not' => 'NOT' },
-                                     -default => $default_not_operator ) . "\n" .
+                  popup_menu(
+                    -name    => 'select_not_operator',
+                    -values  => [ '', 'not' ],
+                    -labels  => { '' => '', 'not' => 'NOT' },
+                    -default => $default_not_operator
+                  )
+                  . "\n"
+                  .
 
-                         popup_menu( -name => 'select_operator',
-                                     -id => 'select_operator',
-                                     -values => [ sort {$Database::filter_operators->{$a}
-                                                        cmp
-                                                        $Database::filter_operators->{$b}}
-                                                  keys %$Database::filter_operators ],
-                                     -labels => $Database::filter_operators,
-                                     -default => [ $default_select_operator ],
-                                     -onchange => 'toggle_search_value()' ) . "\n";
+                  popup_menu(
+                    -name   => 'select_operator',
+                    -id     => 'select_operator',
+                    -values => [
+                        sort {
+                            $Database::filter_operators->{$a}
+                              cmp $Database::filter_operators->{$b}
+                          }
+                          keys %$Database::filter_operators
+                    ],
+                    -labels   => $Database::filter_operators,
+                    -default  => [$default_select_operator],
+                    -onchange => 'toggle_search_value()'
+                  ) . "\n";
             };
-            
-            $html .= textfield( { -id =>'search_text',
-                                  -name => 'search_value',
-                                  -value => $default_search_value }) . "\n";
-            #my $fks_cell_text = $db->get_fks_cell_text( $fk_columns );
-            my $fk_table_data = $db->get_fks_cell_text( $fk_columns );
-            my $ambiguous_fks = {};
-            foreach my $fk( sort keys %{ $fk_table_data->{fk_values} } ){
-                my $fk_table_values = $fk_table_data->{fk_values}{$fk};  
-                $ambiguous_fks->{ $fk } = 
-                    HTMLGenerator::get_ambiguous_fk_values($fk_table_values, $fk, $fk_columns);
+
+            $html .= textfield(
+                {
+                    -id    => 'search_text',
+                    -name  => 'search_value',
+                    -value => $default_search_value
+                }
+            ) . "\n";
+
+            foreach my $fk (@$fk_list) {
+                $html .= qq(<select id="$fk" style="display: none">\n);
+                $html .= qq(</select>\n);
             }
 
-             $html .= HTMLGenerator::convert_fk_to_select( $fk_table_data->{fk_values},
-                                                              $ambiguous_fks,
-                                                              $fk_columns);
+            my $static_fk_dropdown = [
+                map {
+                         $_->name() ne 'revision_id'
+                      && $db->get_count( $_->parent_table() ) <= 100
+                      ? $_
+                      : ()
+                } @$fk_columns
+            ];
+
+            foreach my $fk_column (@$static_fk_dropdown) {
+                my $fk_column_name = $fk_column->name();
+                $html .=
+                    qq(<select id="$fk_column_name" style="display: none">\n);
+                my $parent_table = $fk_column->parent_table();
+                my $fmttable = $db->get_fk_fmttable( $parent_table );
+                my $fk_display_table = defined $fmttable ?
+                    $fmttable : $parent_table;
+
+                my $column_properties =
+                  $db->get_column_properties( $fk_display_table );
+                my $column_sql_types =
+                  $db->get_column_type_hash( $fk_display_table );
+                my @unique_fk_table_columns =
+                  $db->get_unique_columns( $fk_display_table );
+                my $fk_format =
+                  $db->get_fk_format( $fk_display_table );
+                my $dropdown_list = [];
+
+                my $fk_records = $db->get_records(
+                    $fk_display_table,
+                    undef,
+                    {
+                        filter => undef,
+                    }
+                );
+                my $fk_table_id_col =
+                  $db->get_id_column( $fk_display_table );
+
+                foreach my $fk ( @{$fk_records} ) {
+                    my $reversed_data;
+                    $reversed_data->{columns} = {
+                        map {
+                            $_ => {
+
+                                'value'   => $fk->{$_},
+                                'coltype' => $column_properties->{coltype}{$_},
+                                'sqltype' => $column_sql_types->{$_},
+                            }
+
+                        } keys %{$fk}
+                    };
+
+                    $reversed_data->{metadata}{fk_format} = $fk_format;
+                    $reversed_data->{value} = $fk;
+
+                    my $cell_text = HTMLGenerator::get_foreign_key_cell_text(
+                        $fk_column_name,
+                        $fk,
+                        {
+                            record       => $reversed_data,
+                            foreign_keys => $static_fk_dropdown,
+                            ambiguity    => $db->is_uniq_fk($fk_column)
+                        }
+                    );
+
+                    $html .= qq(<option value = "$fk->{$fk_table_id_col}">$cell_text</option>);
+                }
+                $html .= qq(</select>\n);
+            }
+
+
 
             # Set attributes for the radios inside the group.
             my %attributes = (
@@ -1102,7 +1324,7 @@ eval {
                 transfer_query_string(
                     $base_uri,
                     $ENV{REQUEST_URI},
-                    { exclude_re => 'select_.*|search_value|.cgifields|rows|offset',
+                    { exclude_re => 'select_.*|search_value|.cgifields|rows',
                       append => $filter_qs },
                 );
 
@@ -1285,14 +1507,28 @@ eval {
                                                   no_related => 1,
                                               } );
 
+            # If a user has speciied 'depth=...' in the request URI
+            # explicitly, we want to transfer that value to the links
+            # that ponto to individual records (cards); otherwise the
+            # 'depth' QS parameter is not used and the aplication
+            # default depth is chosen:
+            my $record_depth = '';
+            if( $ENV{REQUEST_URI} =~ '(?:\?|&)(depth=\d+)' ) {
+                $record_depth = '?' . $1;
+            }
             my $request_uri =
                 transfer_query_string( $base_uri,
-                                       $base_uri,
-                                       { append => { map { $_ => $params{$_} }
-                                                     grep { $changed->{$_} &&
-                                                            !/^((csv|ods|xlsx?|json)file|spreadsheet)$/ &&
-                                                            !/^column:/ }
-                                                     keys %params } } );
+                                       $base_uri . $record_depth,
+                                       { append => { 
+                                           map { $_ => $params{$_} }
+                                           grep { 
+                                               $changed->{$_} &&
+                                                   !/^((csv|ods|xlsx?|json)file|spreadsheet)$/ &&
+                                                   !/^column:/ 
+                                           }
+                                           keys %params 
+                                         } 
+                                       } );
             my ( $table_html, $new_record_button ) = data2html_list_table(
                 $db,
                 $data,
@@ -1306,7 +1542,6 @@ eval {
                     request_uri => $request_uri,
                     table_properties =>
                         { -class => 'hoverable', -id => 'mydb-table' },
-                    ambiguous_fks => $ambiguous_fks 
                 }
             );
 
@@ -1364,7 +1599,9 @@ eval {
             # bar becomes detached from the data table outer right border.
             # 'row' container is essentially a flex container that can shrink
             # according to its children content.
-            $html .= start_div( {-class => "row"} ) .
+            $html .= start_div( { -class => ( $db->is_view( $db_table ) ?
+                                              "row view" :
+                                              "row" ) } ) .
                      $table_html .
                      end_div();
 
