@@ -20,6 +20,7 @@ use File::Basename qw(basename dirname);
 use HTML::Entities;
 use HTML::FromText;
 use Hash::Merge qw(merge);
+use List::MoreUtils qw(uniq);
 
 use CGIParameters qw(read_cgi_parameters);
 use Database qw( prepare_record_descriptions_for_save_as_new );
@@ -27,7 +28,9 @@ use Database::Order;
 use Database::Filter;
 use HTMLGenerator qw(
     data2html_page
+    data2html_page_block
     data2html_edit_form
+    data2html_edit_form_block
     error_page
     start_html5
     transfer_query_string
@@ -95,11 +98,15 @@ eval {
         Save   => { re => 'Save' },
         'Save as new' => { re => 'Save as new' },
         Delete => { re => 'Delete' },
+        rows => { re => '-1|[0-9]+', default => 10 },
+        offset => { re => '[0-9]+', default => 0 },
         depth => { re => '-1|[0-9]+', default => -1 },
-        format => { re => 'html|csv|xml|cif|json(?:api)?',
+        format => { re => 'html|htmlblock|csv|xml|cif|json(?:api)?',
                     default => 'html' },
         include => { re => '.*' },
 
+        related_table => { re => '.*' },
+
         csvfile  => { re => $filename_re },
         odsfile  => { re => $filename_re },
         xlsfile  => { re => $filename_re },
@@ -177,6 +184,15 @@ eval {
                                                 { key_column => $params{id_column} } );
     my $real_id_column = $db->get_unique_key_column( $db_table, $real_record_id );
 
+    if( !$changed->{rows} && !( $format eq 'html' || $format eq 'htmlblock' ) ) {
+        $params{rows} = -1;
+    }
+
+    # HACK: very nasty hack, but until there are no paginations, the offset has
+    # to be set to 0. Otherwise, empty entry will be shown. However, offset is
+    # necessary and ready for future use.
+    $params{offset} = 0;
+
     my $web_base = dirname dirname $base_uri;
     my $list_uri = dirname( $ENV{REQUEST_URI} );
     if( $ENV{REQUEST_URI} =~ /(\?.*)$/ ) {
@@ -185,7 +201,9 @@ eval {
     }
 
     my $html;
-    my @js_imports = qw( form_validation.js
+    my @js_imports = qw( ajax.form_control.js
+                         form_control.js
+                         form_validation.js
                          jquery.js
                          esc_key_press.js );
     my $js_onload_action;
@@ -215,7 +233,8 @@ eval {
                                        $ENV{REQUEST_URI} ) );
         exit( 0 );
     } elsif( uc($ENV{REQUEST_METHOD}) eq 'POST' && $params{'Edit'} ) {
-        @js_imports = qw( form_control.js
+        @js_imports = qw( ajax.form_control.js
+                          form_control.js
                           form_validation.js
                           jquery.js
                           esc_key_press.js
@@ -224,26 +243,46 @@ eval {
                           dropdown.js );
         $js_onload_action = 'unhide_form_buttons';
 
+        my $requested_tables;
+        if( defined $params{related_table} ) {
+            my %seen_tables = ( $params{related_table} => 1 );
+            my $related_fks = $db->get_foreign_keys( $params{related_table} );
+            my @related_fk_tables = map { $_->parent_table } @$related_fks;
+
+            for my $related_fk_table ( @related_fk_tables ) {
+                next if $seen_tables{$related_fk_table};
+
+                $seen_tables{$related_fk_table} = 1;
+
+                # Secondary FKs are needed for proper vizualization of table
+                # row data.
+                my $second_related_fks = $db->get_foreign_keys( $related_fk_table );
+
+                next if !@$second_related_fks;
+
+                for my $second_related_fk_table ( map { $_->parent_table }
+                                                      @$second_related_fks ) {
+                    $seen_tables{$second_related_fk_table} = 1;
+                }
+            }
+
+            @$requested_tables = keys %seen_tables;
+        }
+
         my $data =
             $db->get_record_descriptions( $db_table,
                                           {
                                               id_column => $real_id_column,
                                               record_id => $real_record_id,
+                                              rows => $params{rows},
+                                              offset => $params{offset},
+                                              depth => $params{depth},
+                                              requested_tables => $requested_tables,
                                               show_fk_values => 1,
                                               show_enumeration_values => 1,
                                               web_base => $web_base,
                                           } );
 
-        my $fk_columns  = $db->get_foreign_keys($db_table);
-        my $fk_table_data = $db->get_fks_cell_text( $fk_columns, 1 );
-        my $ambiguous_fks = {};
-
-        foreach my $fk( sort keys %{ $fk_table_data->{fk_values} } ){
-            my $fk_table_values = $fk_table_data->{fk_values}{$fk};
-            $ambiguous_fks->{ $fk } =
-             HTMLGenerator::get_ambiguous_fk_values($fk_table_values, $fk, $fk_columns);
-        }
-
         my $buttons =
             submit( { -name  => 'Update',
                       -value => 'Update',
@@ -263,20 +302,37 @@ eval {
                     -class => button_classes_string('secondary') )
                     . "\n" .
             end_form . "\n";
-        $html .= data2html_edit_form(  $db, $data, $db_table,
-                                  {
-                                      request_uri => $ENV{REQUEST_URI},
-                                      level => 2,
-                                      vertical => 1,
-                                      add_button_as_link => 1,
-                                      table_properties =>
+        if( $format eq 'htmlblock' ) {
+            $html .= data2html_edit_form_block(
+                        $db, $data, $db_table,
+                        {
+                            request_uri => $ENV{REQUEST_URI},
+                            level => 2,
+                            vertical => 1,
+                            add_button_as_link => 1,
+                            table_properties =>
+                            {
+                                -class => 'record-hoverable dbdata-input-form'
+                            },
+                        }
+            );
+        } else {
+            $html .= data2html_edit_form(  $db, $data, $db_table,
                                       {
-                                          -class => 'record-hoverable dbdata-input-form',
-                                      },
-                                      form_head => $buttons,
-                                      form_tail => br . $buttons,
-                                      ambiguous_fks => $ambiguous_fks
-                                  } );
+                                          request_uri => $ENV{REQUEST_URI},
+                                          rows => $params{rows},
+                                          offset => $params{offset},
+                                          level => 2,
+                                          vertical => 1,
+                                          add_button_as_link => 1,
+                                          table_properties =>
+                                          {
+                                              -class => 'record-hoverable dbdata-input-form',
+                                          },
+                                          form_head => $buttons,
+                                          form_tail => br . $buttons,
+                                      } );
+        }
     } elsif( uc($ENV{REQUEST_METHOD}) eq 'DELETE' || $params{Delete} ) {
         # Delete the database record:
         $db->delete_record( $real_record_id, $db_table,
@@ -326,6 +382,9 @@ eval {
         if( defined $real_record_id && !$params{"Save"} ) {
             $get_record_descriptions_options->{id_column} = $real_id_column;
             $get_record_descriptions_options->{record_id} = $real_record_id;
+            $get_record_descriptions_options->{rows}  = $params{rows};
+            $get_record_descriptions_options->{offset}  = $params{offset};
+            $get_record_descriptions_options->{depth}  = $params{depth};
             $get_record_descriptions_options->{no_empty}  = 1;
         } else {
             $get_record_descriptions_options->{template} = 1;
@@ -482,6 +541,32 @@ eval {
 
     if(  uc($ENV{REQUEST_METHOD}) ne 'DELETE' &&
         (uc($ENV{REQUEST_METHOD}) ne 'POST' || (!$params{Edit} && !$params{Delete})) ) {
+        my $requested_tables;
+        if( defined $params{related_table} ) {
+            my %seen_tables = ( $params{related_table} => 1 );
+            my $related_fks = $db->get_foreign_keys( $params{related_table} );
+            my @related_fk_tables = map { $_->parent_table } @$related_fks;
+
+            for my $related_fk_table ( @related_fk_tables ) {
+                next if $seen_tables{$related_fk_table};
+
+                $seen_tables{$related_fk_table} = 1;
+
+                # Secondary FKs are needed for proper vizualization of table
+                # row data.
+                my $second_related_fks = $db->get_foreign_keys( $related_fk_table );
+
+                next if !@$second_related_fks;
+
+                for my $second_related_fk_table ( map { $_->parent_table }
+                                                      @$second_related_fks ) {
+                    $seen_tables{$second_related_fk_table} = 1;
+                }
+            }
+
+            @$requested_tables = keys %seen_tables;
+        }
+
         # Show the database record:
         my $data =
             $db->get_record_descriptions( $db_table,
@@ -490,6 +575,9 @@ eval {
                                               depth => $params{depth},
                                               filter => $filter,
                                               order => $order,
+                                              rows => $params{rows},
+                                              offset => $params{offset},
+                                              requested_tables => $requested_tables,
                                               no_empty => 1,
                                               web_base => $web_base,
                                           } );
@@ -524,16 +612,6 @@ eval {
             print "\n";
         } elsif( $format eq 'html' ) {
             my $filter = Database::Filter->new( { filter => $params->{filter} } );
-            
-            my $fk_columns  = $db->get_foreign_keys($db_table);
-            my $fk_table_data = $db->get_fks_cell_text( $fk_columns, 1 );
-            my $ambiguous_fks = {};
-
-            foreach my $fk( sort keys %{ $fk_table_data->{fk_values} } ){
-                my $fk_table_values = $fk_table_data->{fk_values}{$fk};
-                $ambiguous_fks->{ $fk } =
-                HTMLGenerator::get_ambiguous_fk_values($fk_table_values, $fk, $fk_columns);
-            }
 
             # Collect record data preprint html.
             # Variables set to split to page elements.
@@ -548,6 +626,8 @@ eval {
                                     id_column => $params{id_column},
                                     record_id => $real_record_id,
                                     order => $order,
+                                    rows => $params{rows},
+                                    offset => $params{offset},
                                     filter => $filter,
                                     level => 2,
                                     vertical => 1,
@@ -556,9 +636,6 @@ eval {
                                         -class => 'record-hoverable'
                                     },
                                     request_uri => $ENV{REQUEST_URI},
-                                    rows => $params{rows},
-                                    offset => $params{offset},
-                                    ambiguous_fks => $ambiguous_fks
                                 } );
 
             $two_column_row_html_options = {
@@ -569,6 +646,25 @@ eval {
             };
 
             $html = $record_modification_buttons . $table_html;
+        } elsif ( $format eq 'htmlblock' ) {
+            # TODO: a bit of 'html' format's code duplication.
+            my $filter = Database::Filter->new( { filter => $params->{filter} } );
+            $html = data2html_page_block(
+                        $db, $data, $db_table,
+                        {
+                            id_column => $params{id_column},
+                            record_id => $real_record_id,
+                            order => $order,
+                            filter => $filter,
+                            level => 2,
+                            vertical => 1,
+                            table_properties =>
+                            {
+                                -class => 'record-hoverable'
+                            },
+                            request_uri => $ENV{REQUEST_URI},
+                        }
+            );
         } else {
             InputException->throw( "Unsupported format '$format'" );
         }
@@ -646,68 +742,77 @@ eval {
         }
 
         # This ends the HTML page.
-        $html = $cgi->header( -type => 'text/html',
-                              -expires => 'now',
-                              -charset => 'UTF-8',
-                              -status  => $HTTP_status,
-                              ($location ? (-location => $location ) : ()) ) .
-            start_html5( $cgi, { -title => "$record_id record - $db_table table - $db_name db: " .
-                                           $RestfulDB::Defaults::website_title,
-                                 -head => Link({
-                                                -rel  => 'icon',
-                                                -type => 'image/x-icon',
-                                                -href => '../../images/favicon.ico'
-                                 }),
-                                 -meta => {
-                                   'viewport' => 'width=device-width, initial-scale=1',
-                                 },
-                                 -script => \@script,
-                                 -style => get_css_styles( $db_name, $page_level ) } ) .
+        if( $format eq 'htmlblock' ) {
+            $html = $cgi->header( -type => 'text/html',
+                                  -expires => 'now',
+                                  -charset => 'UTF-8',
+                                  -status  => $HTTP_status,
+                                  ($location ? (-location => $location ) : ()) ) .
+                    $html;
+        } else {
+            $html = $cgi->header( -type => 'text/html',
+                                  -expires => 'now',
+                                  -charset => 'UTF-8',
+                                  -status  => $HTTP_status,
+                                  ($location ? (-location => $location ) : ()) ) .
+                start_html5( $cgi, { -title => "$record_id record - $db_table table - $db_name db: " .
+                                               $RestfulDB::Defaults::website_title,
+                                     -head => Link({
+                                                    -rel  => 'icon',
+                                                    -type => 'image/x-icon',
+                                                    -href => '../../images/favicon.ico'
+                                     }),
+                                     -meta => {
+                                       'viewport' => 'width=device-width, initial-scale=1',
+                                     },
+                                     -script => \@script,
+                                     -style => get_css_styles( $db_name, $page_level ) } ) .
 
-            # The beginning of the HTML page.
-            start_root() .
-            header_record_pl( $db_name,
-                              $db_table,
-                              $selection_filter_string,
-                              $record_id ) .
-            start_doc_wrapper() .
-            start_main_content() .
+                # The beginning of the HTML page.
+                start_root() .
+                header_record_pl( $db_name,
+                                  $db_table,
+                                  $selection_filter_string,
+                                  $record_id, { 'is_view' => $db->is_view( $db_table ) } ) .
+                start_doc_wrapper() .
+                start_main_content() .
 
-            # Hidden 'top' paragraph on the data page for the 'To top' link.
-            p( { -id => 'top' }, 'Top' ) . "\n" .
+                # Hidden 'top' paragraph on the data page for the 'To top' link.
+                p( { -id => 'top' }, 'Top' ) . "\n" .
 
-            # Warnings
-            warnings2html( @warnings ) .
+                # Warnings
+                warnings2html( @warnings ) .
 
-            # Start record navigation card.
-            start_card_fluid( { class => 'record-navigation-panel' } ) .
-            # Setting record_id to reside in major column displays record id info
-            # in a single centered column.
-            two_column_row_html( {
-                column_major => "Record id: $record_id",
-                row_class => 'page-title'
-            } ) .
+                # Start record navigation card.
+                start_card_fluid( { class => 'record-navigation-panel' } ) .
+                # Setting record_id to reside in major column displays record id info
+                # in a single centered column.
+                two_column_row_html( {
+                    column_major => "Record id: $record_id",
+                    row_class => 'page-title'
+                } ) .
 
-            two_column_row_html( $two_column_row_html_options ) .
-            end_card() . # Close the first card.
+                two_column_row_html( $two_column_row_html_options ) .
+                end_card() . # Close the first card.
 
-            # Start the main data card.
-            start_card_fluid() .
-            start_div( { -class => 'section' } ) . "\n" .
+                # Start the main data card.
+                start_card_fluid() .
+                start_div( { -class => 'section' } ) . "\n" .
 
-            $html;
+                $html;
+
+            $html .= end_div() . "\n"; # Close the first section.
+            $html .= end_card(); # Close the second card.
+            $html .= footer_record_pl( $db_name,
+                                       $db_table,
+                                       $selection_filter_string,
+                                       $record_id);
+            $html .= end_main_content();
+            $html .= end_doc_wrapper();
+            $html .= end_root();
+            $html .= end_html . "\n";
+        }
 
-        $html .= end_div() . "\n"; # Close the first section.
-        $html .= end_card(); # Close the second card.
-        $html .= footer_record_pl( $db_name,
-                                   $db_table,
-                                   $selection_filter_string,
-                                   $record_id);
-        $html .= end_main_content();
-        $html .= end_doc_wrapper();
-        $html .= end_root();
-        $html .= end_html . "\n";
-        
         print $html;
     }
 
