@@ -593,7 +593,7 @@ function uuid() {
     });
 }
 
-function search_suggest( fk_list ){
+function search_suggest( fk_list, fk_ajax_list ){
     $('#search_text').val('');
     var select_column = $("[name='select_column']").val();
     $('#search_text').show();
@@ -623,8 +623,52 @@ function search_suggest( fk_list ){
             $('#search_text').attr('name','');
             $('#'+fk).attr('name','search_value');
             
-            if( !$('#'+fk).hasClass("select2-hidden-accessible") ) {
-                $('#'+fk).select2();
+            if( !$('#'+fk).hasClass("select2-hidden-accessible") && fk_ajax_list.includes(fk) ) {
+                $('#'+fk).select2({
+                    ajax: {
+                        url: window.location.href,
+                        data: function(params){
+                            var query = {
+                                format: 'json',
+                                ajax: 'select2',
+                                fk_column: fk,
+                                search: params.term,
+                                page: params.page || 0
+                            }
+                            return query;
+                        },
+                        dataType: 'json',
+                        delay: 250,
+                        processResults: function(obj){
+                            var resData = [];
+                            var data = obj.data;
+                            data.forEach(function(value){
+                                resData.push(value)
+                            })
+                            return{
+                                results: $.map(resData, function(item){
+                                    return {
+                                        text: item.text,
+                                        id: item.id
+                                    }
+                                }),
+                                pagination: obj.pagination
+                            };
+                        },
+                        cache: true
+                    },
+                    //minimumInputLength: 1,
+                    placeholder: "Select "+fk,
+                    allowClear: true,
+                    width: 'style'
+                });
+            } else {
+                $('#'+fk).select2({
+                    placeholder: "Select "+fk,
+                    allowClear: true,
+                    width: 'style'
+                });
+                $('#'+fk).val(null).trigger('change');
             }
         } else {
             $('#'+fk).attr('name','');
